#include "ros/ros.h"
#include "std_msgs/String.h"

#include "geometry_msgs/Twist.h"

#include "turtlesim/Pose.h"

#define PI 3.14159265359

//global publisher for velocity
ros::Publisher velocity_pub;

//the turtle's current pose gets stored here
turtlesim::Pose current_pose;

//the x and y coordinates of the points the turtle has to visit
//these are C++ arrays. You can access them by x_coords[i] where
//i is an integer in the range of 0-9. 
float x_coords[] = {5.3,1.0,4.5,5.5,2.3,6.6,4.6,7.8,3.7,8.7};
float y_coords[] = {6.5,2.1,9.8,3.4,7.8,8.9,2.1,8.1,2.3,4.7};

void pose_cb(const turtlesim::Pose::ConstPtr& msg){
	current_pose = *msg;
}

int main(int argc, char **argv){
	
	//initialize the node
	ros::init(argc, argv, "traveling_salesturtle");
	
	//instantiate the node handle which is used for creating publishers and subscribers
	ros::NodeHandle n;
	
	//publisher for velocity commands
	velocity_pub = n.advertise<geometry_msgs::Twist>("/turtle1/cmd_vel", 1);

	//subscriber for the turtle's pose
	ros::Subscriber sub_pose = n.subscribe("/turtle1/pose", 1, pose_cb );
   
	float distance_traveled = 0.0;
	float travel_time = 0.0;
	
	//your code for tsolving the traveling salesmane problem goes here
	
	/* Each time your turtle arrives at the i^th point,
	 *  the following code should be executed:
	 * 
	 * ROS_INFO("I arrived at point %i.", current_point_index);
	 * 
	 * The int variable "current_point_index" should specify the index (starting at 0) of the point in the global arrays that the turtle just visited. E.g., if the point is (1.0,2.1) then the current_point_index should be 1
	 * 
	 */
	
	//at the end, these should be printed to the console
	ROS_INFO("Total distance traveled: %f",distance_traveled);
	ROS_INFO("Total traveled time: %f",travel_time);
	
	return 0;
}
