//include ROS
#include "ros/ros.h"

//include the header file for the Twist message (used to specify velocity commands)
#include "geometry_msgs/Twist.h"

//include the header file for the turtlesim/Pose message (what the simulator publishes)
#include "turtlesim/Pose.h"

//TO DO: include the  header for our own .srv file
#include "first_package_cs378/TurtleCommand.h"

#define PI 3.14159265359

//global publisher for the turtle's velocity
ros::Publisher velocity_pub;

//global variable storing the turtle's current pose
turtlesim::Pose current_pose;

void pose_cb(const turtlesim::Pose::ConstPtr& msg){
	current_pose = *msg;
}

bool closeEnough(float angle1, float angle2, float eps){
	float d1 = fabs(angle1 - angle2);
	float d2 = fabs(angle1 - angle2);
	
	ROS_INFO("%f, %f",d1,d2);
	
	if (d1 < eps || d2 > (2*PI - eps)){
		return true;
	}
	else {
		return false;
	}
}

void turn(float angular_vel, float desired_orientation){
	geometry_msgs::Twist v;
	
	v.linear.x = 0.0;
	v.linear.y = 0.0;
	v.linear.z = 0.0;
	v.angular.x = 0.0;
	v.angular.y = 0.0;
	v.angular.z = angular_vel;
	
	float framerate = 40.0;
	ros::Rate loop_rate(framerate);
	
	bool reached = false;
	
	while (!reached){
		//publish the velocity command
		velocity_pub.publish(v);
		
		//listen for incomming messages
		ros::spinOnce();
		
		if (closeEnough(desired_orientation,  
						current_pose.theta,PI/10.0)){
			reached = true;				
		}
		
		loop_rate.sleep();
	}
}

void moveForward(float v_x, float duration){
	geometry_msgs::Twist v;
	
	v.linear.x = v_x;
	v.linear.y = 0.0;
	v.linear.z = 0.0;
	v.angular.x = 0.0;
	v.angular.y = 0.0;
	v.angular.z = 0.0;

	float framerate = 40.0;
	ros::Rate loop_rate(framerate);
	
	
	for (float t = 0; t < duration; t += 1.0/framerate){
		velocity_pub.publish(v);
		loop_rate.sleep();
		ROS_INFO("Moving forward....");
	}
}


bool service_callback(
		first_package_cs378::TurtleCommand::Request  &req,
        first_package_cs378::TurtleCommand::Response &res)
{
	ROS_INFO("I got called!");
	ROS_INFO("The command I received is %s", req.command.c_str());
	
	if (req.command == "forward"){
		ROS_INFO("The parameter is: %f",req.param);
		moveForward(0.5,1.0);
		res.result = true;
	}
  
  return true;
}

int main(int argc, char **argv){
	
	//declare our node to ROS
	ros::init(argc, argv, "turtle_controller_server");
	ros::NodeHandle n;
	
	//create the publisher for command velocities
	velocity_pub = n.advertise<geometry_msgs::Twist>("/turtle1/cmd_vel", 1);

	//subscribe to the turtle's position and orientation 
	ros::Subscriber sub_pose = n.subscribe("/turtle1/pose", 1, pose_cb );
   
	//TO DO: declare a service and write the callback function
    ros::ServiceServer service = n.advertiseService("/turtle_controller_server/command", service_callback);
    
   
   
    //finally, just spin (i.e., listen for service requests and messages)
	ros::spin();
	
	
	return 0;
}
